#pragma semicolon 1

#define MAX_SAYTXT_LENGTH 192

#include <sourcemod>
#include <smlib>
#include "morecolors.inc"

#define PLUGIN_VERSION "0.9"
#define SOUND_BLIP      "buttons/blip1.wav"
#define SOUND_GO        "hl1/fvox/beep.wav"
#define SOUND_ERROR     "buttons/button10.wav"
#define PAUSE_MSGPREFIX "{orangered}||"

new unpauseDelay;
new bool:onpause = false;
new bool:unpause_delayed = false;
new bool:pauseCmdDelaying = false;

new Handle:g_hPausable = INVALID_HANDLE;
new Handle:g_hSpectatorCanPause = INVALID_HANDLE;
new Handle:g_hLetInOnPause = INVALID_HANDLE;
new Handle:g_hAfterpauseDelay = INVALID_HANDLE;
new Handle:g_hUnpauseDelay = INVALID_HANDLE;
new Handle:g_hPauseOnGameEnd = INVALID_HANDLE;

public Plugin:myinfo =
{
    name = "Advanced hl2dm pause",
    author = "hl2dm.net",
    description = "Advanced pause for competitive gameplay with chat fix",
    version = PLUGIN_VERSION,
    url = "http://hl2dm.net/"
};

public OnPluginStart()
{
    AddCommandListener(Command_Pause, "pause");
    g_hPausable = FindConVar("sv_pausable");
    g_hSpectatorCanPause = CreateConVar("sm_specpause", "0", "Determines if spectators can use pause or not", FCVAR_NOTIFY, true, 0.0, true, 1.0);
    g_hLetInOnPause = CreateConVar("sm_letinonpause", "1", "Determines if players can join on paused server. Such clients will quickly toggle pause so they won't stuck on loading screen", FCVAR_NOTIFY, true, 0.0, true, 1.0);
    g_hAfterpauseDelay = CreateConVar("sm_AfterpauseDelay", "2", "Sets the delay being used to ignore pause commands from clients righ after the game was paused.", FCVAR_NOTIFY, true, 0.0, false, _);
    g_hUnpauseDelay = CreateConVar("sm_UnpauseDelay", "3", "The amount of time in seconds used by countdown. Setting this to 0 disables countdown.", FCVAR_NOTIFY, true, 0.0, false, _);
    g_hPauseOnGameEnd = CreateConVar("sm_pauseongameend", "0", "Determines if pause command is availible after the game ended", FCVAR_NOTIFY, true, 0.0, true, 1.0);

    AutoExecConfig();
}

public OnConfigsExecuted()
{
    unpauseDelay = GetConVarInt(g_hUnpauseDelay) -1;

    if (!GetConVarBool(g_hPauseOnGameEnd))
    {
        HookUserMessage(GetUserMessageId("VGUIMenu"), Hook_VGUIMenu);
    }
}

public Action:OnClientSayCommand(client, const String:command[], const String:sArgs[])
{
    if (!onpause)
    {
        return Plugin_Continue;
    }
    decl String:clientName[MAX_NAME_LENGTH];
    GetClientName(client, clientName, sizeof(clientName));
    decl String:messageArg[MAX_SAYTXT_LENGTH];
    strcopy(messageArg, sizeof(messageArg), sArgs);
    StripQuotes(messageArg);
    decl clientTeam;
    clientTeam = GetClientTeam(client);
    if (StrEqual(command, "say_team", false))
    {
        for (new i=1; i<=MaxClients; i++)
        {
            if(!IsClientInGame(i))
            {
                continue;
            }
            else if (clientTeam == GetClientTeam(i))
            {
                if (clientTeam == 1) //spectator
                {
                    CPrintToChatEx(i, client,"%s {default}(Spectator) {teamcolor}%s{default} : %s", PAUSE_MSGPREFIX, clientName, messageArg);
                }
                else
                {
                    CPrintToChatEx(i, client,"%s {default}(TEAM) {teamcolor}%s{default} : %s", PAUSE_MSGPREFIX, clientName, messageArg);
                }
            }
        }
        return Plugin_Handled;
    }
    else if (StrEqual(command, "say", false))
    {
        if (clientTeam == 1) //spectator
        {
            CPrintToChatAllEx(client,"%s {default}*SPEC* {teamcolor}%s{default} : %s", PAUSE_MSGPREFIX, clientName, messageArg);
        }
        else
        {
            CPrintToChatAllEx(client,"%s {teamcolor}%s{default} : %s", PAUSE_MSGPREFIX, clientName, messageArg);
        }
        return Plugin_Handled;
    }
    return Plugin_Continue;
}

public Action:Hook_VGUIMenu(UserMsg:msg_id, Handle:msg, const players[], playersNum, bool:reliable, bool:init)
{
    new String:s[10];
    BfReadString(msg, s, sizeof(s));
    if (strcmp(s, "scores", false) == 0)
    {
        pauseCmdDelaying = true;
        return Plugin_Continue;
    }
    return Plugin_Continue;
}

public OnMapEnd()
{
    onpause = false;
    unpause_delayed = false;
    pauseCmdDelaying = false;
}

public OnClientDisconnect_Post(client)
{
    if(Client_GetCount(true, false) == 0)
    {
        onpause = false;
        unpause_delayed = false;
        pauseCmdDelaying = false;
    }

}

public OnClientPutInServer(client)
{
    if(!onpause || !GetConVarBool(g_hLetInOnPause))
    {

    }
    else
    {
        unpause_delayed = true;
        FakeClientCommand(client, "pause");
        CreateTimer(0.1, wait01, GetClientUserId(client));
    }
}

public Action:wait01(Handle:timer, any:usrID)
{
    new client = GetClientOfUserId(usrID);
    if (client == 0)
    {
        return Plugin_Continue;
    }
    FakeClientCommand(client, "pause");
    return Plugin_Continue;
}


public Action:Command_Pause(client, const String:command[], args)
{
    if(!GetConVarBool(g_hPausable) || pauseCmdDelaying || (GetClientTeam(client)==1 && !GetConVarBool(g_hSpectatorCanPause)))
    {
        return Plugin_Handled;
    }
    decl String:clientName[64];
    GetClientName(client, clientName, sizeof(clientName));
    if (!onpause)
    {
        CPrintToChatAll("{aliceblue}[SM] {purple}%s{aliceblue} paused the game",clientName);
        onpause = true;
        unpause_delayed = false;
        pauseCmdDelaying = true;
        CreateTimer(GetConVarFloat(g_hAfterpauseDelay), StopDelayingPauseCmd);
        return Plugin_Continue;
    }
    else if (!unpause_delayed)
    {
        if (unpauseDelay<0)
        {
            onpause=false;
            return Plugin_Continue;
        }
        PrintCenterTextAll("%i",GetConVarInt(g_hUnpauseDelay));
        PlaySoundAll(SOUND_BLIP);
        CreateTimer(1.0, Unpause_Countdown, GetClientUserId(client), TIMER_REPEAT);
        CPrintToChatAll("{aliceblue}[SM] The game will be unpaused in %i seconds by {purple}%s",GetConVarInt(g_hUnpauseDelay), clientName);
        pauseCmdDelaying = true;
        return Plugin_Handled;
    }
    onpause=false;
    unpause_delayed=false;
    return Plugin_Continue;
}

public Action:Unpause_Countdown(Handle:timer, any:usrID)
{
    new client = GetClientOfUserId(usrID);
    if (unpauseDelay==0)
    {
        PrintCenterTextAll("");
        if (IsClientInGame(client))
        {
            PlaySoundAll(SOUND_GO);
            unpauseDelay=GetConVarInt(g_hUnpauseDelay)-1;
            pauseCmdDelaying = false;
            unpause_delayed = true;
            FakeClientCommand(client, "pause");
            return Plugin_Stop;
        }
        CPrintToChatAll("{aliceblue}[SM] Failed to unpause the game, because the \"unpauser\" left the game");
        PlaySoundAll(SOUND_ERROR);
        unpauseDelay=GetConVarInt(g_hUnpauseDelay)-1;
        pauseCmdDelaying = false;
        return Plugin_Stop;
    }
    PrintCenterTextAll("%i",unpauseDelay);
    PlaySoundAll(SOUND_BLIP);
    unpauseDelay--;
    return Plugin_Continue;
}

public Action:StopDelayingPauseCmd(Handle:timer)
{
    pauseCmdDelaying = false;
}

PlaySoundAll(String:sndpath[])
{
    for (new i = 1; i <= MaxClients; i++)
    {
        if(!IsClientInGame(i))
        {
            continue;
        }
        ClientCommand(i, "playgamesound \"%s\"", sndpath);
    }
}