/*****************************************************************


C O M P I L E   O P T I O N S


*****************************************************************/
// enforce semicolons after each code statement
#pragma semicolon 1

/*****************************************************************


P L U G I N   I N C L U D E S


*****************************************************************/
#include <sourcemod>
#include <sdktools>
#include <smlib>
#include <colors>
#include <smlib/pluginmanager>

/*****************************************************************


P L U G I N   I N F O


*****************************************************************/
#define PLUGIN_NAME				"Connect Spectate"
#define PLUGIN_TAG				"sm"
#define PLUGIN_AUTHOR			"Chanz"
#define PLUGIN_DESCRIPTION		"This plugin sends everyone into spectate as soon they are connected"
#define PLUGIN_VERSION 			"2.10.22"
#define PLUGIN_URL				"http://forums.alliedmods.net/showthread.php?p=1375084 OR http://www.mannisfunhouse.eu/"

public Plugin:myinfo = {
	name = PLUGIN_NAME,
	author = PLUGIN_AUTHOR,
	description = PLUGIN_DESCRIPTION,
	version = PLUGIN_VERSION,
	url = PLUGIN_URL
}

/*****************************************************************


		P L U G I N   D E F I N E S


*****************************************************************/
#define TEAM_UNASSIGNED	0
#define TEAM_SPECTATOR	1
#define TEAM_BLUE		2
#define TEAM_RED		3

/*****************************************************************


		G L O B A L   V A R S


*****************************************************************/
//ConVar
new Handle:g_cvarPlayerTrigger 	= INVALID_HANDLE;
new Handle:g_cvarTeamPlay		= INVALID_HANDLE;

//ConVar Runtime Optimizer
new g_iPlugin_PlayerTrigger = 0;
new g_iPlugin_TeamPlay = -1;

//Misc
new bool:g_bRemoveRagDollOnDeath[MAXPLAYERS+1];
new Float:g_fNextModelChangeTime[MAXPLAYERS+1];

new String:g_sTeamName[4][64];

/*****************************************************************


		F O R W A R D   P U B L I C S


*****************************************************************/
public OnPluginStart() {
	
	//Init for smlib
	SMLib_OnPluginStart(PLUGIN_NAME,PLUGIN_TAG,PLUGIN_VERSION,PLUGIN_AUTHOR,PLUGIN_DESCRIPTION,PLUGIN_URL);
	
	//Translations (you should use it always when printing something to clients)
	//Always with the short name and .phrases postfix.
	decl String:translationsName[PLATFORM_MAX_PATH];
	Format(translationsName,sizeof(translationsName),"%s.phrases",g_sPlugin_Short_Name);
	File_LoadTranslations(translationsName);
	
	//ConVar
	g_cvarPlayerTrigger = CreateConVarEx("playertrigger","2","Only send players into spectate if there are already playing more players than this value",FCVAR_PLUGIN,true,0.0);
	g_cvarTeamPlay = FindConVar("mp_teamplay");
	
	//ConVar Runtime Optimizer
	g_iPlugin_PlayerTrigger = GetConVarInt(g_cvarPlayerTrigger);
	g_iPlugin_TeamPlay = (g_cvarTeamPlay != INVALID_HANDLE) ? GetConVarInt(g_cvarTeamPlay) : -1;
	
	//ConVar Hooks (To update runtime optimizer)
	HookConVarChange(g_cvarPlayerTrigger,ConVarChange_PlayerTrigger);
	//Dont hook teamplay since it should change only on mapchange!
	
	AddCommandListener(Command_JoinTeam,"jointeam");
	
	//Console Cmd
	RegConsoleCmd("sm_jointeam",Command_Sm_JoinTeam,"hooks jointeam and adds the chat trigger for Connect Spectate plugin");
	
	
	//Event Hooks
	HookEvent("player_death",Event_Player_Death);
	
	//UserMsg Hook
	HookUserMessage(GetUserMessageId("TextMsg"), MsgHook_TextMsg, true);
	
	//Auto Config (you should always use it)
	//Always with "plugin." prefix and the short name
	decl String:configName[MAX_PLUGIN_SHORTNAME_LENGTH+8];
	Format(configName,sizeof(configName),"plugin.%s",g_sPlugin_Short_Name);
	AutoExecConfig(true,configName);
}

public OnMapStart() {
	
	// hax against valve fail (thx psychonic for fix)
	if(GuessSDKVersion() == SOURCE_SDK_EPISODE2VALVE){
		SetConVarString(g_cvarVersion, PLUGIN_VERSION);
	}
	
	g_iPlugin_TeamPlay = (g_cvarTeamPlay != INVALID_HANDLE) ? GetConVarInt(g_cvarTeamPlay) : -1;
	
	//TeamNames don't call before mapstart!
	GetTeamName(TEAM_UNASSIGNED,g_sTeamName[TEAM_UNASSIGNED],sizeof(g_sTeamName[]));
	GetTeamName(TEAM_SPECTATOR,g_sTeamName[TEAM_SPECTATOR],sizeof(g_sTeamName[]));
	GetTeamName(TEAM_BLUE,g_sTeamName[TEAM_BLUE],sizeof(g_sTeamName[]));
	GetTeamName(TEAM_RED,g_sTeamName[TEAM_RED],sizeof(g_sTeamName[]));
}

public OnClientConnected(client){
	
	Client_Init(client);
}

public OnClientPostAdminCheck(client){
	
	Client_Init(client);
}

public OnClientPutInServer(client){
	
	Handle_FirstJoin(client,GetClientTeam(client));
}

/****************************************************************


		C A L L B A C K   F U N C T I O N S


****************************************************************/
public ConVarChange_PlayerTrigger(Handle:cvar, const String:oldVal[], const String:newVal[]){

	g_iPlugin_PlayerTrigger = StringToInt(newVal);
}


public Action:Event_Player_Death(Handle:event, const String:name[], bool:dontBroadcast){
	
	if(g_iPlugin_Enable == 0){
		return Plugin_Continue;
	}
	
	new client = GetClientOfUserId(GetEventInt(event,"userid"));
	
	if(!Client_IsValid(client) || IsFakeClient(client)){
		return Plugin_Continue;
	}
	
	if(g_bRemoveRagDollOnDeath[client]){
		
		g_bRemoveRagDollOnDeath[client] = false;
		
		new ragdoll = GetEntPropEnt(client, Prop_Send, "m_hRagdoll");
		if ((ragdoll < 0) || !IsValidEntity(ragdoll)){
			return Plugin_Continue;
		}
		Server_PrintDebug("[%s] Removing the ragdoll (%d) of client %N",PLUGIN_NAME,ragdoll,client);
		RemoveEdict(ragdoll);
	}
	return Plugin_Continue;
}

Action:Handle_FirstJoin(client,team,newTeam=-1){
	
	if(!Client_IsValid(client) || IsFakeClient(client)){
		return Plugin_Continue;
	}
	
	if(g_iPlugin_PlayerTrigger+1 > Client_GetCount()){
		Server_PrintDebug("client '%N' is allowed to change team at wish because the PlayerTrigger (%d) is higher than the player count(%d)",client,g_iPlugin_PlayerTrigger,Client_GetCount());
		return Plugin_Continue;
	}
	
	PrintToServer("client: %N joined the first time in team: %d",client,team);
	
	if(newTeam == TEAM_SPECTATOR){
		
		return Plugin_Continue;
	}
	
	if(team == TEAM_SPECTATOR){
		
		return Plugin_Continue;
	}
	
	CPrintToChat(client,"%t","Type jointeam into chat");
	FakeClientCommandEx(client,"jointeam %i", TEAM_SPECTATOR);
	g_bRemoveRagDollOnDeath[client] = true;
	
	return Plugin_Continue;
}

public Action:Command_JoinTeam(client, const String:command[], argc){
	
	return Handle_JoinTeam(client,command,argc);
}

public Action:Command_Sm_JoinTeam(client,args){
	
	decl String:command[64];
	
	GetCmdArg(0,command,sizeof(command));
	
	return Handle_JoinTeam(client,command,args);
}

public Action:MsgHook_TextMsg(UserMsg:msg_id, Handle:bf, const players[], playersNum, bool:reliable, bool:init) {
	
	new String:name[256];
	new destination = BfReadByte(bf);
	BfReadString(bf, name, sizeof(name));
	
	new String:str1[] = "Please wait";
	new String:str2[] = "joined team";
	
	if (destination == 3 && playersNum == 1) {
		
		if (strncmp(name, str1, strlen(str1)) == 0 || strncmp(name, str2, strlen(str2)) == 0) {
			
			return Plugin_Handled;
		}
	}
	
	return Plugin_Continue;
}


public Handle_JoinMenu(Handle:menu, MenuAction:action, client, param) {
	
	if (action == MenuAction_Select) {
		
		decl String:info[16];
		
		if(GetMenuItem(menu, param, info, sizeof(info))){
			
			switch(StringToInt(info)){
				
				case 0:	FakeClientCommand(client,"jointeam %i", TEAM_UNASSIGNED);
				case 1:	FakeClientCommand(client,"jointeam %i", TEAM_SPECTATOR);
				case 2:	FakeClientCommand(client,"jointeam %i", TEAM_BLUE);
				case 3: FakeClientCommand(client,"jointeam %i", TEAM_RED);
			}
		}
	}
	else if(action == MenuAction_End){
		
		CloseHandle(menu);
	}
}
/*****************************************************************


		P L U G I N   F U N C T I O N S


*****************************************************************/
stock Action:Handle_JoinTeam(client, const String:command[], argc){
	
	if(g_iPlugin_Enable == 0){
		return Plugin_Continue;
	}
	
	if(!Client_IsValid(client) || IsFakeClient(client)){
		return Plugin_Continue;
	}
	
	if(g_fNextModelChangeTime[client] >= GetGameTime()){
		
		CPrintToChat(client,"{default}Please wait {green}%d {default}more seconds before trying to switch.",RoundToCeil(g_fNextModelChangeTime[client]-GetGameTime()));
		return Plugin_Handled;
	}
	
	new String:arg[11];
	GetCmdArgString(arg,sizeof(arg));
	
	Server_PrintDebug("[%s] client %N used command jointeam %s",PLUGIN_NAME,client,arg);
	
	new newTeam = StringToInt(arg);
	new team = GetClientTeam(client);
	
	if(((newTeam == 0) && StrEqual(arg,"",false)) || !Math_InBounds(newTeam,0,3)){
		
		//Player doesn't know in what team he wants? let him choose by menu!
		Menu_ShowJoinTeam(client);
		return Plugin_Handled;
	}
	
	Client_PrintDebug(client,"you want to change team from %d to %d",team, newTeam);
	
	new bool:valveFail = false;
	
	switch(g_iPlugin_TeamPlay){
		
		case 0:{
			
			//this is just for a nice chat message.
			if((newTeam == TEAM_BLUE) || (newTeam == TEAM_RED)){
				
				newTeam = TEAM_UNASSIGNED;
			}
		}
		case 1:{
			
			if(newTeam == TEAM_UNASSIGNED){
				
				newTeam = TEAM_RED;
				valveFail = true;
			}
		}
	}
	
	if(team != newTeam){
		
		if(team != TEAM_SPECTATOR){
			
			if(IsPlayerAlive(client)){
				
				g_bRemoveRagDollOnDeath[client] = true;
				Client_RemoveAllWeapons(client,"",true);
				ForcePlayerSuicide(client);
				Client_SetScore(client,Client_GetScore(client)+1);
				Client_SetDeaths(client,Client_GetDeaths(client)-1);
			}
		}
		
		ChangeClientTeam(client,newTeam);
		
		//fix for valve fail that you can't join the game with "jointeam 0" when teamplay is 0
		//or when teamplay is 1 and you'd like to join the game with "jointeam 0", when spectator is impossible.
		if((newTeam == TEAM_UNASSIGNED) || valveFail){
			
			DispatchSpawn(client);
		}
		
		if(newTeam != TEAM_SPECTATOR){
			
			g_fNextModelChangeTime[client] = GetGameTime()+6.0;
		}
		
		decl String:clientAuth[MAX_STEAMAUTH_LENGTH];
		GetClientAuthString(client,clientAuth,sizeof(clientAuth));
		
		CPrintToChatAll("{green}%N (%s) {default}joined team {green}%s",client,clientAuth,g_sTeamName[newTeam]);
	}
	else {
		
		CPrintToChat(client,"{default}You are already in team {green}%s.",g_sTeamName[team]);
	}
	
	return Plugin_Continue;
}

/* 
 * Returns if the value is within the given bounds (min & max).
 * 
 * @param value		The value you want to check.
 * @param min		The lower border.
 * @param max		The upper border.
 * @return			True if the value is within bounds (bigger or equal min / smaller or equal max), other wise false.
 */
stock bool:Math_InBounds(any:value,any:min,any:max){
	
	if((value < min) || (value > max)){
		
		return false;
	}
	
	return true;
}

Menu_ShowJoinTeam(client) {
	
	new Handle:menu = CreateMenu(Handle_JoinMenu);
	SetMenuTitle(menu, "%t", "Choose a team");
	
	AddMenuItem(menu, "1", g_sTeamName[TEAM_SPECTATOR]);
	
	switch(g_iPlugin_TeamPlay){
		
		case 0:{
			AddMenuItem(menu, "0", g_sTeamName[TEAM_UNASSIGNED]);
		}
		case 1:{
			AddMenuItem(menu, "2", g_sTeamName[TEAM_BLUE]);
			AddMenuItem(menu, "3", g_sTeamName[TEAM_RED]);
		}
	}
	DisplayMenu(menu, client, 60);
}

//This function will be called within SMLib_OnPluginStart.
stock ClientAll_Init(){
	
	for(new client=1;client<=MaxClients;client++){
		
		if(!IsClientInGame(client)){
			continue;
		}
		
		Client_Init(client);
	}
}

stock Client_Init(client){
	
	//Variables
	Client_InitVars(client);
	
	//Functions
}

stock Client_InitVars(client){
	
	//PluginVars
	g_bRemoveRagDollOnDeath[client] = false;
	g_fNextModelChangeTime[client] = GetGameTime();
}


