/*****************************************************************


C O M P I L E   O P T I O N S


*****************************************************************/
// enforce semicolons after each code statement
#pragma semicolon 1

/*****************************************************************


P L U G I N   I N C L U D E S


*****************************************************************/
#include <sourcemod>
#include <sdktools>
#include <smlib>


/*****************************************************************


P L U G I N   I N F O


*****************************************************************/
#define PLUGIN_NAME				"Fix HL2DM TeamScore TeamKill"
#define PLUGIN_TAG				"sm"
#define PLUGIN_AUTHOR			"Chanz"
#define PLUGIN_DESCRIPTION		"Fixes the issue that the team score goes up by 1 point for every team kill."
#define PLUGIN_VERSION 			"1.0.0"
#define PLUGIN_URL				"<url>"

public Plugin:myinfo = {
	name = PLUGIN_NAME,
	author = PLUGIN_AUTHOR,
	description = PLUGIN_DESCRIPTION,
	version = PLUGIN_VERSION,
	url = PLUGIN_URL
}

/*****************************************************************


		P L U G I N   D E F I N E S


*****************************************************************/


/*****************************************************************


		G L O B A L   V A R S


*****************************************************************/
//Use a good notation, constants for arrays, initialize everything that has nothing to do with clients!
//If you use something which requires client index init it within the function Client_InitVars (look below)
//Example: Bad: "decl servertime" Good: "new g_iServerTime = 0"
//Example client settings: Bad: "decl saveclientname[33][32] Good: "new g_szClientName[MAXPLAYERS+1][MAX_NAME_LENGTH];" -> later in Client_InitVars: GetClientName(client,g_szClientName,sizeof(g_szClientName));

//ConVars
new Handle:g_cvarTeamPlay = INVALID_HANDLE;
new Handle:g_cvarFriendlyFire = INVALID_HANDLE;
new Handle:g_cvarRemovePoints = INVALID_HANDLE;

//ConVar Runtime Optimizer
new g_iPlugin_TeamPlay = -1;
new g_iPlugin_FriendlyFire = -1;
new g_iPlugin_RemovePoints = 0;
/*****************************************************************


		F O R W A R D   P U B L I C S


*****************************************************************/
public OnPluginStart() {
	
	//Init for smlib
	SMLib_OnPluginStart(PLUGIN_NAME,PLUGIN_TAG,PLUGIN_VERSION,PLUGIN_AUTHOR,PLUGIN_DESCRIPTION,PLUGIN_URL);
	
	decl String:gameFolder[PLATFORM_MAX_PATH];
	GetGameFolderName(gameFolder,sizeof(gameFolder));
	if(!StrEqual(gameFolder,"hl2mp",false)){
		SetFailState("this game/mod isn't hl2mp, so you probably don't need this plugin, but if you think you need this plugin for you game/mod contact %s on %s, and ask him to add it",PLUGIN_AUTHOR,PLUGIN_URL);
	}
	
	//Command Hooks (AddCommandListener) (If the command already exists, like the command kill, then hook it!)
	
	
	//Register New Commands (RegConsoleCmd) (If the command doesn't exist, hook it here)
	
	
	//Register Admin Commands (RegAdminCmd)
	
	
	//Cvars: Create a global handle variable (HookConVarChange is automatically added).
	//Example: g_cvarEnable = CreateConVarEx("enable","1","example ConVar");
	g_cvarRemovePoints = CreateConVarEx("remove_teampoints","2","how many points do you want to remove from the team score when a team kill occur?",FCVAR_PLUGIN|FCVAR_NOTIFY);
	g_cvarTeamPlay = FindConVar("mp_teamplay");
	g_cvarFriendlyFire = FindConVar("mp_friendlyfire");
	
	//Set your ConVar runtime optimizers here
	//Example: g_iPlugin_Enable = GetConVarInt(g_cvarEnable);
	g_iPlugin_RemovePoints = GetConVarInt(g_cvarRemovePoints);
	g_iPlugin_TeamPlay = (g_cvarTeamPlay != INVALID_HANDLE) ? GetConVarInt(g_cvarTeamPlay) : -1;
	g_iPlugin_FriendlyFire = (g_cvarFriendlyFire != INVALID_HANDLE) ? GetConVarInt(g_cvarFriendlyFire) : -1;
	
	//Event Hooks
	new bool:hookEvent_Player_Death = HookEventEx("player_death",Event_Player_Death,EventHookMode_Pre);
	if(!hookEvent_Player_Death){
		SetFailState("can't hook player_death event, contact %s on %s, to ask why and how to fix",PLUGIN_AUTHOR,PLUGIN_URL);
	}
	
	//Auto Config (you should always use it)
	//Always with "plugin." prefix and the short name
	decl String:configName[MAX_PLUGIN_SHORTNAME_LENGTH+8];
	Format(configName,sizeof(configName),"plugin.%s",g_sPlugin_Short_Name);
	AutoExecConfig(true,configName);
}

public OnMapStart() {
	
	// hax against valvefail (thx psychonic for fix)
	if(GuessSDKVersion() == SOURCE_SDK_EPISODE2VALVE){
		SetConVarString(g_cvarVersion, PLUGIN_VERSION);
	}
	
	g_iPlugin_TeamPlay = (g_cvarTeamPlay != INVALID_HANDLE) ? GetConVarInt(g_cvarTeamPlay) : -1;
}


/****************************************************************


		C A L L B A C K   F U N C T I O N S


****************************************************************/
public SMLib_ConVarChange(Handle:cvar, const String:oldVal[], const String:newVal[]){
	

	//Setting runtime optimizer
	//SMLib ConVars (don't remove)
	if(cvar == g_cvarEnable){
		g_iPlugin_Enable = StringToInt(newVal);
	}
	
	else if(cvar == g_cvarDebug){
		g_iPlugin_Debug = StringToInt(newVal);
	}
	
	else if(cvar == g_cvarDeveloper_Flag){
		g_afPlugin_Developer_Flag = Admin_GetFlagByChar(newVal);
	}
	
	//Additional Plugin ConVars
	
	else if(cvar == g_cvarFriendlyFire){
		g_iPlugin_FriendlyFire = StringToInt(newVal);
	}
	
	else if(cvar == g_cvarRemovePoints){
		g_iPlugin_FriendlyFire = StringToInt(newVal);
	}
	
	else if(cvar == g_cvarTeamPlay){
		//can't set teamplay here since it only take effect on map change
	}
	
	//Error Handling (don't remove)
	else {
		
		decl String:cvarName[64];
		GetConVarName(cvar,cvarName,sizeof(cvarName));
		LogError("SMLib_ConVarChange was called because cvar '%s' changed from '%s' to '%s', but there isn't an runtime optimizer!",cvarName,oldVal,newVal);
	}
}


public Action:Event_Player_Death(Handle:event, const String:name[], bool:dontBroadcast){
	
	if(g_iPlugin_Enable < 1){
		return Plugin_Continue;
	}
	
	if(g_iPlugin_RemovePoints == 0){
		Server_PrintDebug("cvar remove_teampoints is 0 -> plugin_continue");
		return Plugin_Continue;
	}
	
	if(g_iPlugin_TeamPlay != 1){
		Server_PrintDebug("teamplay not enabled -> plugin_continue");
		return Plugin_Continue;
	}
	
	if(g_iPlugin_FriendlyFire != 1){
		Server_PrintDebug("friendly fire not enabled -> plugin_continue");
		return Plugin_Continue;
	}
	
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	new attacker = GetClientOfUserId(GetEventInt(event, "attacker"));
	
	if(!Client_IsValid(client) || !Client_IsValid(attacker)){
		Server_PrintDebug("client %d or attacker %d isn't valid -> plugin_continue",client,attacker);
		return Plugin_Continue;
	}
	
	if(client == attacker){
		Server_PrintDebug("client %d killed him self -> plugin_continue",client);
		return Plugin_Continue;
	}
	
	new clientTeam = GetClientTeam(client);
	new attackerTeam = GetClientTeam(attacker);
	
	if(clientTeam != attackerTeam){
		Server_PrintDebug("client and attacker aren't in the same team -> plugin_continue");
		return Plugin_Continue;
	}
	
	new teamScore = GetTeamScore(clientTeam);
	
	SetTeamScore(clientTeam,teamScore-g_iPlugin_RemovePoints);
	
	return Plugin_Continue;
}
/*****************************************************************


		P L U G I N   F U N C T I O N S


*****************************************************************/



